use std::{fs::File, io::BufReader};

use log::warn;
use serde::Deserialize;

use super::{deserialize, table_directory::{FontTable, TableDirectoryRecord}};

#[derive(Debug, Deserialize, Default)]
pub struct Version {
	major: u16,
	minor: u16
}

#[derive(Debug, Deserialize)]
pub struct FontHeader {
	#[serde(skip)]
	pub version: Version,
	
	#[serde(skip)]
	pub font_revision: Version,

	checksum_adjust: u32,
	magic_number: u32,
	pub flags: u16,
	pub units_per_em: u16,

	pub created: i64,
	pub modified: i64,

	pub xmin: i16,
	pub ymin: i16,
	pub xmax: i16,
	pub ymax: i16,

	pub mac_style: u16,
	pub lowest_rec_ppem: u16,

	pub font_direction_hint: i16,
	pub index_to_loc_format: i16,
	pub glyph_data_format: i16
}

impl FontTable for FontHeader {
	const TAG: &'static str = "head";
	type InitData = ();

	fn decode(reader: BufReader<File>, _: &TableDirectoryRecord, _: Option<Self::InitData>) -> bincode::Result<Self>
		where Self: Sized
	{
		let version = deserialize::<_, Version>(reader.get_ref())?;
		if version.major != 1 && version.minor != 0 {
			warn!("Font version is not 1.0");
		}

		let font_revision = deserialize::<_, Version>(reader.get_ref())?;

		let mut font_header: FontHeader = deserialize(reader.get_ref())?;
		font_header.version = version;
		font_header.font_revision = font_revision;

		Ok(font_header)
	}
}
