use std::fmt::Debug;

use log::debug;
use serde::Deserialize;

#[derive(Debug, Copy, Clone, Deserialize, Default)]
pub struct SearchParameters {
	pub seg_count_x2: u16,		// num of elements x2
	pub search_range: u16,		// largest power of two smaller than num of elements
	pub entry_selector: u16,	// max levels in binary search
	pub _range_shift: u16		// remaining elements in array
}

impl SearchParameters {
	fn binary_search(&self, array: &[u16], target: u16) -> Option<usize> {
		let mut index = (self.search_range >> 2) as usize;
		for level in 1..=self.entry_selector {
			let end_code = array[index];

			if end_code >= target && array[index - 1] < target {
				debug!("binary search found result");
				return Some(index);
			}

			let mut correction = (self.search_range >> (level + 2)) as usize;
			if correction == 0 {
				correction = 1;
			}

			if end_code < target {
				index += correction;
				debug!("considered endcode smaller than target, setting index to {index}");	
			} else {
				let new_index = index - correction;
				if correction == 1 && array[new_index] < target {
					debug!("no more candidates");	
					return Some(index);
				}

				index = new_index;
				debug!("considered endcode larger than target, setting index to {index}");	
			}
		}

		Some(index + 1)
	}

	pub fn find_first_goe(&self, array: &[u16], target: u16) -> Option<usize> {
		if target > array[(self.seg_count_x2 as usize >> 1) - 1] {
			return None;
		}

		if target > array[(self.search_range as usize >> 1) - 1] {
			debug!("{}", array[(self.search_range as usize >> 1) - 1]);
			
			for i in (self.search_range as usize >> 1)..array.len() {
				if array[i] >= target {
					return Some(i);
				}
			}
		}

		self.binary_search(array, target)
	}
}
