mod json;

use crate::font::{Glyph, GlyphHeader, GlyphPoint, SplineElement};

use log::debug;
pub use json::JsonWriter;

pub trait Visitor: Sized {
	fn write_prefix(&mut self, header: &GlyphHeader);

	fn write_point(&mut self, point: &GlyphPoint);
	fn write_spline(&mut self, spline: &SplineElement);
	fn write_suffix(self);

	fn write(mut self, glyph: &Glyph) {
		self.write_prefix(&glyph.header);

		glyph.iter().for_each(|point| {
			self.write_point(&point);
		});

		glyph.splines().unwrap().for_each(|spline_element| {
			debug!("{spline_element:#?}");
			self.write_spline(&spline_element)
		});

		debug!("done writing splines");

		self.write_suffix();
	}
}
